@@ -11,6 +11,7 @@ from fsspec import AbstractFileSystem, filesystem
 from fsspec.callbacks import _DEFAULT_CALLBACK
 from fsspec.compression import compr
 from fsspec.core import BaseCache, MMapCache
+from fsspec.exceptions import BlocksizeMismatchError
 from fsspec.spec import AbstractBufferedFile
 from fsspec.utils import infer_compression
 
@@ -24,8 +25,9 @@ class CachingFileSystem(AbstractFileSystem):
     access after the initial download. The files are stored in a given
     directory with random hashes for the filenames. If no directory is given,
     a temporary one is used, which should be cleaned up by the OS after the
-    process ends. The files themselves as sparse (as implemented in
-    MMapCache), so only the data which is accessed takes up space.
+    process ends. The files themselves are sparse (as implemented in
+    :class:`~fsspec.caching.MMapCache`), so only the data which is accessed
+    takes up space.
 
     Restrictions:
 
@@ -87,10 +89,13 @@ class CachingFileSystem(AbstractFileSystem):
             decompression.
         """
         super().__init__(**kwargs)
+        if fs is None and target_protocol is None:
+            raise ValueError(
+                "Please provide filesystem instance(fs) or target_protocol"
+            )
         if not (fs is None) ^ (target_protocol is None):
             raise ValueError(
-                "Please provide one of filesystem instance (fs) or"
-                " remote_protocol, not both"
+                "Both filesystems (fs) and target_protocol may not be both given."
             )
         if cache_storage == "TMP":
             storage = [tempfile.mkdtemp()]
@@ -159,7 +164,13 @@ class CachingFileSystem(AbstractFileSystem):
                     if c["blocks"] is True or cache[k]["blocks"] is True:
                         c["blocks"] = True
                     else:
-                        c["blocks"] = set(c["blocks"]).union(cache[k]["blocks"])
+                        # self.cached_files[*][*]["blocks"] must continue to
+                        # point to the same set object so that updates
+                        # performed by MMapCache are propagated back to
+                        # self.cached_files.
+                        blocks = cache[k]["blocks"]
+                        blocks.update(c["blocks"])
+                        c["blocks"] = blocks
                     c["time"] = max(c["time"], cache[k]["time"])
                     c["uid"] = cache[k]["uid"]
 
@@ -173,8 +184,8 @@ class CachingFileSystem(AbstractFileSystem):
         for c in cache.values():
             if isinstance(c["blocks"], set):
                 c["blocks"] = list(c["blocks"])
-        fn2 = tempfile.mktemp()
-        with open(fn2, "wb") as f:
+        fd2, fn2 = tempfile.mkstemp()
+        with open(fd2, "wb") as f:
             pickle.dump(cache, f)
         self._mkcache()
         move(fn2, fn)
@@ -229,9 +240,10 @@ class CachingFileSystem(AbstractFileSystem):
         raises PermissionError
         """
         path = self._strip_protocol(path)
-        _, fn = self._check_file(path)
-        if fn is None:
+        details = self._check_file(path)
+        if not details:
             return
+        _, fn = details
         if fn.startswith(self.storage[-1]):
             # is in in writable cache
             os.remove(fn)
@@ -308,7 +320,7 @@ class CachingFileSystem(AbstractFileSystem):
             block_size=block_size,
             autocommit=autocommit,
             cache_options=cache_options,
-            cache_type=None,
+            cache_type="none",
             **kwargs,
         )
         if self.compression:
@@ -320,7 +332,7 @@ class CachingFileSystem(AbstractFileSystem):
             f = compr[comp](f, mode="rb")
         if "blocksize" in detail:
             if detail["blocksize"] != f.blocksize:
-                raise ValueError(
+                raise BlocksizeMismatchError(
                     "Cached file must be reopened with same block"
                     "size as original (old: %i, new %i)"
                     "" % (detail["blocksize"], f.blocksize)
@@ -343,7 +355,7 @@ class CachingFileSystem(AbstractFileSystem):
         path = self._strip_protocol(f.path)
 
         c = self.cached_files[-1][path]
-        if c["blocks"] is not True and len(["blocks"]) * f.blocksize >= f.size:
+        if c["blocks"] is not True and len(c["blocks"]) * f.blocksize >= f.size:
             c["blocks"] = True
         try:
             logger.debug("going to save")
@@ -485,8 +497,7 @@ class WholeFileCacheFileSystem(CachingFileSystem):
             self._mkcache()
         else:
             return [
-                LocalTempFile(self.fs, path, mode=open_files.mode, autocommit=False)
-                for path in paths
+                LocalTempFile(self.fs, path, mode=open_files.mode) for path in paths
             ]
 
         if self.compression:
@@ -529,6 +540,13 @@ class WholeFileCacheFileSystem(CachingFileSystem):
 
     def commit_many(self, open_files):
         self.fs.put([f.fn for f in open_files], [f.path for f in open_files])
+        [f.close() for f in open_files]
+        for f in open_files:
+            # in case autocommit is off, and so close did not already delete
+            try:
+                os.remove(f.name)
+            except FileNotFoundError:
+                pass
 
     def _make_local_details(self, path):
         hash = self.hash_name(path, self.same_names)
@@ -574,7 +592,8 @@ class WholeFileCacheFileSystem(CachingFileSystem):
         out = {}
         callback.set_size(len(paths))
         for p, fn in zip(paths, fns):
-            out[p] = open(fn, "rb").read()
+            with open(fn, "rb") as f:
+                out[p] = f.read()
             callback.relative_update(1)
         if isinstance(path, str) and len(paths) == 1 and recursive is False:
             out = out[paths[0]]
@@ -583,7 +602,7 @@ class WholeFileCacheFileSystem(CachingFileSystem):
     def _open(self, path, mode="rb", **kwargs):
         path = self._strip_protocol(path)
         if "r" not in mode:
-            return self.fs._open(path, mode=mode, **kwargs)
+            return LocalTempFile(self, path, mode=mode)
         detail = self._check_file(path)
         if detail:
             detail, fn = detail
@@ -623,7 +642,7 @@ class WholeFileCacheFileSystem(CachingFileSystem):
                 f = compr[comp](f, mode="rb")
                 data = True
                 while data:
-                    block = getattr(f, "blocksize", 5 * 2 ** 20)
+                    block = getattr(f, "blocksize", 5 * 2**20)
                     data = f.read(block)
                     f2.write(data)
         else:
@@ -703,7 +722,7 @@ class SimpleCacheFileSystem(WholeFileCacheFileSystem):
                 f = compr[comp](f, mode="rb")
                 data = True
                 while data:
-                    block = getattr(f, "blocksize", 5 * 2 ** 20)
+                    block = getattr(f, "blocksize", 5 * 2**20)
                     data = f.read(block)
                     f2.write(data)
         else:
@@ -715,10 +734,13 @@ class LocalTempFile:
     """A temporary local file, which will be uploaded on commit"""
 
     def __init__(self, fs, path, fn=None, mode="wb", autocommit=True, seek=0):
-        fn = fn or tempfile.mktemp()
+        if fn:
+            self.fn = fn
+            self.fh = open(fn, mode)
+        else:
+            fd, self.fn = tempfile.mkstemp()
+            self.fh = open(fd, mode)
         self.mode = mode
-        self.fn = fn
-        self.fh = open(fn, mode)
         if seek:
             self.fh.seek(seek)
         self.path = path
@@ -740,6 +762,8 @@ class LocalTempFile:
         self.close()
 
     def close(self):
+        if self.closed:
+            return
         self.fh.close()
         self.closed = True
         if self.autocommit:
@@ -751,6 +775,15 @@ class LocalTempFile:
 
     def commit(self):
         self.fs.put(self.fn, self.path)
+        try:
+            os.remove(self.fn)
+        except (PermissionError, FileNotFoundError):
+            # file path may be held by new version of the file on windows
+            pass
+
+    @property
+    def name(self):
+        return self.fn
 
     def __getattr__(self, item):
         return getattr(self.fh, item)
