@@ -10,6 +10,7 @@ from urllib.parse import urlparse
 
 import aiohttp
 import requests
+import yarl
 
 from fsspec.asyn import AbstractAsyncStreamedFile, AsyncFileSystem, sync, sync_wrapper
 from fsspec.callbacks import _DEFAULT_CALLBACK
@@ -53,6 +54,7 @@ class HTTPFileSystem(AsyncFileSystem):
         loop=None,
         client_kwargs=None,
         get_client=get_client,
+        encoded=False,
         **storage_options,
     ):
         """
@@ -90,6 +92,7 @@ class HTTPFileSystem(AsyncFileSystem):
         self.cache_options = cache_options
         self.client_kwargs = client_kwargs or {}
         self.get_client = get_client
+        self.encoded = encoded
         self.kwargs = storage_options
         self._session = None
 
@@ -104,6 +107,9 @@ class HTTPFileSystem(AsyncFileSystem):
         request_options.pop("skip_instance_cache", None)
         self.kwargs = request_options
 
+    def encode_url(self, url):
+        return yarl.URL(url, encoded=self.encoded)
+
     @staticmethod
     def close_session(loop, session):
         if loop is not None and loop.is_running():
@@ -143,7 +149,7 @@ class HTTPFileSystem(AsyncFileSystem):
         kw.update(kwargs)
         logger.debug(url)
         session = await self.set_session()
-        async with session.get(url, **self.kwargs) as r:
+        async with session.get(self.encode_url(url), **self.kwargs) as r:
             self._raise_not_found_for_status(r, url)
             text = await r.text()
         if self.simple_links:
@@ -217,7 +223,7 @@ class HTTPFileSystem(AsyncFileSystem):
             headers["Range"] = await self._process_limits(url, start, end)
             kw["headers"] = headers
         session = await self.set_session()
-        async with session.get(url, **kw) as r:
+        async with session.get(self.encode_url(url), **kw) as r:
             out = await r.read()
             self._raise_not_found_for_status(r, url)
         return out
@@ -229,7 +235,7 @@ class HTTPFileSystem(AsyncFileSystem):
         kw.update(kwargs)
         logger.debug(rpath)
         session = await self.set_session()
-        async with session.get(rpath, **self.kwargs) as r:
+        async with session.get(self.encode_url(rpath), **self.kwargs) as r:
             try:
                 size = int(r.headers["content-length"])
             except (ValueError, KeyError):
@@ -297,7 +303,7 @@ class HTTPFileSystem(AsyncFileSystem):
         try:
             logger.debug(path)
             session = await self.set_session()
-            r = await session.get(path, **kw)
+            r = await session.get(self.encode_url(path), **kw)
             async with r:
                 return r.status < 400
         except (requests.HTTPError, aiohttp.ClientError):
@@ -342,7 +348,7 @@ class HTTPFileSystem(AsyncFileSystem):
         if block_size and size:
             return HTTPFile(
                 self,
-                path,
+                self.encode_url(path),
                 session=session,
                 block_size=block_size,
                 mode=mode,
@@ -354,7 +360,12 @@ class HTTPFileSystem(AsyncFileSystem):
             )
         else:
             return HTTPStreamFile(
-                self, path, mode=mode, loop=self.loop, session=session, **kw
+                self,
+                self.encode_url(path),
+                mode=mode,
+                loop=self.loop,
+                session=session,
+                **kw,
             )
 
     async def open_async(self, path, mode="rb", size=None, **kwargs):
@@ -365,7 +376,12 @@ class HTTPFileSystem(AsyncFileSystem):
             except FileNotFoundError:
                 pass
         return AsyncStreamFile(
-            self, path, loop=self.loop, session=session, size=size, **kwargs
+            self,
+            self.encode_url(path),
+            loop=self.loop,
+            session=session,
+            size=size,
+            **kwargs,
         )
 
     def ukey(self, url):
@@ -389,7 +405,7 @@ class HTTPFileSystem(AsyncFileSystem):
             try:
                 info.update(
                     await _file_info(
-                        url,
+                        self.encode_url(url),
                         size_policy=policy,
                         session=session,
                         **self.kwargs,
@@ -604,7 +620,7 @@ class HTTPFile(AbstractBufferedFile):
         kwargs = self.kwargs.copy()
         headers = kwargs.pop("headers", {}).copy()
         headers["Range"] = "bytes=%i-%i" % (start, end - 1)
-        logger.debug(self.url + " : " + headers["Range"])
+        logger.debug(str(self.url) + " : " + headers["Range"])
         r = await self.session.get(self.url, headers=headers, **kwargs)
         async with r:
             if r.status == 416:
