@@ -108,6 +108,16 @@ def netloc(ssh):
     return userpass + netloc
 
 
+def test_put_file(ssh, tmp_path, root_path):
+
+    tmp_file = tmp_path / "a.txt"
+    with open(tmp_file, mode="w") as fd:
+        fd.write("blabla")
+
+    f = fsspec.get_filesystem_class("sftp")(**ssh)
+    f.put_file(lpath=tmp_file, rpath=root_path + "a.txt")
+
+
 def test_simple_with_tar(ssh, netloc, tmp_path, root_path):
 
     files_to_pack = ["a.txt", "b.txt"]
@@ -162,6 +172,21 @@ def test_transaction(ssh, root_path):
         f.rm(root_path, recursive=True)
 
 
+def test_mkdir_create_parent(ssh):
+    f = fsspec.get_filesystem_class("sftp")(**ssh)
+
+    with pytest.raises(FileNotFoundError):
+        f.mkdir("/a/b/c")
+
+    f.mkdir("/a/b/c", create_parents=True)
+    assert f.exists("/a/b/c")
+
+    with pytest.raises(FileExistsError, match="/a/b/c"):
+        f.mkdir("/a/b/c")
+
+    f.rm("/a/b/c", recursive=True)
+
+
 def test_makedirs_exist_ok(ssh):
     f = fsspec.get_filesystem_class("sftp")(**ssh)
 
