@@ -3,7 +3,6 @@
 import contextlib
 import os
 import pickle
-import sys
 import tempfile
 
 import pytest
@@ -40,7 +39,7 @@ def test_class_methods():
     assert MemoryFileSystem._get_kwargs_from_urls("memory://user@thing") == {}
 
 
-def test_get_put(tmpdir):
+def test_get_put(tmpdir, m):
     tmpdir = str(tmpdir)
     fn = os.path.join(tmpdir, "one")
     open(fn, "wb").write(b"one")
@@ -94,19 +93,21 @@ def test_get_put(tmpdir):
     assert open(fn, "rb").read() == b"one"
 
 
-def test_du():
+def test_du(m):
     fs = MemoryFileSystem()
-    fs.store = {
-        "/dir/afile": MemoryFile(fs, "/afile", b"a"),
-        "/dir/dirb/afile": MemoryFile(fs, "/afile", b"bb"),
-        "/dir/dirb/bfile": MemoryFile(fs, "/afile", b"ccc"),
-    }
+    fs.store.update(
+        {
+            "/dir/afile": MemoryFile(fs, "/afile", b"a"),
+            "/dir/dirb/afile": MemoryFile(fs, "/afile", b"bb"),
+            "/dir/dirb/bfile": MemoryFile(fs, "/afile", b"ccc"),
+        }
+    )
     assert fs.du("/dir") == 6
     assert fs.du("/dir", total=False)["/dir/dirb/afile"] == 2
     assert fs.du("/dir", maxdepth=0) == 1
 
 
-def test_head_tail():
+def test_head_tail(m):
     fs = MemoryFileSystem()
     with fs.open("/myfile", "wb") as f:
         f.write(b"I had a nice big cabbage")
@@ -114,7 +115,7 @@ def test_head_tail():
     assert fs.tail("/myfile", 7) == b"cabbage"
 
 
-def test_move():
+def test_move(m):
     fs = MemoryFileSystem()
     with fs.open("/myfile", "wb") as f:
         f.write(b"I had a nice big cabbage")
@@ -124,7 +125,7 @@ def test_move():
     assert isinstance(fs.ukey("/otherfile"), str)
 
 
-def test_recursive_get_put(tmpdir):
+def test_recursive_get_put(tmpdir, m):
     fs = MemoryFileSystem()
     os.makedirs(f"{tmpdir}/nest")
     for file in ["one", "two", "nest/other"]:
@@ -140,7 +141,7 @@ def test_recursive_get_put(tmpdir):
             f.read() == b"data"
 
 
-def test_pipe_cat():
+def test_pipe_cat(m):
     fs = MemoryFileSystem()
     fs.pipe("afile", b"contents")
     assert fs.cat("afile") == b"contents"
@@ -150,7 +151,7 @@ def test_pipe_cat():
     assert fs.cat(list(data)) == data
 
 
-def test_read_block_delimiter():
+def test_read_block_delimiter(m):
     fs = MemoryFileSystem()
     with fs.open("/myfile", "wb") as f:
         f.write(b"some\n" b"lines\n" b"of\n" b"text")
@@ -162,7 +163,7 @@ def test_read_block_delimiter():
     assert fs.read_block("/myfile", 0, None) == fs.cat("/myfile")
 
 
-def test_open_text():
+def test_open_text(m):
     fs = MemoryFileSystem()
     with fs.open("/myfile", "wb") as f:
         f.write(b"some\n" b"lines\n" b"of\n" b"text")
@@ -170,6 +171,17 @@ def test_open_text():
     assert f.encoding == "latin1"
 
 
+def test_read_text(m):
+    with m.open("/myfile", "w", encoding="utf-8") as f:
+        f.write("some\nlines\nof\ntext")
+    assert m.read_text("/myfile", encoding="utf-8") == "some\nlines\nof\ntext"
+
+
+def test_write_text(m):
+    m.write_text("/myfile", "some\nlines\nof\ntext", encoding="utf-8")
+    assert m.read_text("/myfile", encoding="utf-8") == "some\nlines\nof\ntext"
+
+
 def test_chained_fs():
     d1 = tempfile.mkdtemp()
     d2 = tempfile.mkdtemp()
@@ -213,7 +225,6 @@ def test_multilevel_chained_fs():
             assert f.read().decode("utf-8") == f.name
 
 
-@pytest.mark.skipif(sys.version_info < (3, 7), reason="no seek in old zipfile")
 def test_multilevel_chained_fs_zip_zip_file():
     """This test reproduces fsspec/filesystem_spec#334"""
     import zipfile
